% The function of active contour (chanvese)
function seg = Active_Contour(Raw_image, mask, dia_particle)

scale = 1.0;
I = imresize(Raw_image, scale);
P = double(I);
dia_particle = scale * dia_particle;

% Set mask
mask = imresize(mask, scale);
% mask = maskcircle2(I,'whole'); 
% mask = mask(:,:,1);

% Set parameters
layer = 1;
phi0 = bwdist(mask)-bwdist(1-mask)+im2double(mask)-.5;
mu = 0.2; 

num_iter = 200; % The maximum number of iteration
%-- Main loop
for n=1:num_iter
    inidx = find(phi0>=0); % frontground index
    outidx = find(phi0<0); % background index
    force_image = 0; % initial image force for each layer
    for i=1:layer
        L = im2double(P(:,:,i)); % get one image component
        c1 = sum(sum(L.*Heaviside(phi0)))/(length(inidx)+eps); % average inside of Phi0
        c2 = sum(sum(L.*(1-Heaviside(phi0))))/(length(outidx)+eps); % verage outside of Phi0
        force_image=-(L-c1).^2+(L-c2).^2+force_image;
        % sum Image Force on all components (used for vector image)
        % if 'chan' is applied, this loop become one sigle code as a
        % result of layer = 1
    end
    
    % calculate the external force of the image
    force = mu*kappa(phi0)./max(max(abs(kappa(phi0))))+1/layer.*force_image;
    
    % normalized the force
    force = force./(max(max(abs(force))));
    
    % get stepsize dt
    dt = 10.0 * (1.0 - 0.5 *(1.0 - double(dia_particle) / 60.0));
    
    % get parameters for checking whether to stop
    old = phi0;
    phi0 = phi0+dt.*force;
    new = phi0;
    indicator = checkstop(old,new,dt); %check whether to stop
    
    % intermediate output
%     if(mod(n,50) == 0)
%         fprintf('The %dth iteration of Active Contour...\n', n);
%         % showphi(I,phi0,n);
%     end;
    if indicator % decide to stop or continue
        %showphi(I,phi0,n);
        
        %make mask from SDF
        seg = phi0<=0; %-- Get mask from levelset     
        % imshow(seg); title('Global Region-Based Segmentation');
        
        return;
    end
end;

% Reach the maximum iteration
% showphi(I,phi0,n);
%make mask from SDF
seg = phi0<=0; %-- Get mask from levelset
% imshow(seg); title('Global Region-Based Segmentation');