function [Ell_Score] = Calculate_Score_Ell(Ell_Para, Ell_Contour, Raw_image, dia_particle)

[n_height, n_width] = size(Raw_image);
num_particle = length(Ell_Contour);
Ell_Score = zeros(1, num_particle);

% 0 for background, 1 for non-overlappped and 2 for overlapped
Mask_image = zeros(n_height, n_width); 
% The No. of the nano-particles
No_image = zeros(n_height, n_width);

for Inx = 1:num_particle
    x = Ell_Para(Inx).x;
    y = Ell_Para(Inx).y;
    a = Ell_Para(Inx).a;
    b = Ell_Para(Inx).b;
    phi = Ell_Para(Inx).phi;
    orientation_rad = phi;
    cos_phi = cos( orientation_rad );
    sin_phi = sin( orientation_rad );
    R = [ cos_phi sin_phi; -sin_phi cos_phi];
    Center_or = R' * [x; y];
    X0 = Center_or(1);
    Y0 = Center_or(2);
   
    Boundary = Ell_Contour{Inx};
    for m = min(Boundary(:, 1)) : max(Boundary(:, 1))
        if m <= 1 || m >= n_height
            continue;
        end
        for n = min(Boundary(:, 2)) : max(Boundary(:, 2))
            if n <= 1 || n >= n_width
                continue;
            end
            cor_untiled = R' * [n; m];
            n_off = cor_untiled(1) - X0;
            m_off = cor_untiled(2) - Y0;
            
            r_ell = a^2* b^2 * (n_off^2 + m_off^2);
            dis_now = (n_off^2 + m_off^2) * (b^2 * n_off^2 + a^2 * m_off^2);
            
            if dis_now <= r_ell
                if Mask_image(m, n) == 0
                    Mask_image(m, n) = 1;
                    No_image(m, n) = Inx;
                elseif Mask_image(m, n) == 1
                    Mask_image(m, n) = 2;
                    No_image(m, n) = 0;
                end
            end
        end
    end
end

dia_back = round(double(dia_particle) / 3.0); 
Back_image = (Mask_image > 0);
SE = strel('disk', dia_back);
Back_image = imdilate(Back_image, SE);

[~, Assign] = bwdist(No_image > 0);
Label_image = zeros(n_height, n_width); % Assign label to each pixel
for m = 1:n_height
    for n = 1:n_width
        if Back_image(m, n) == 0
            continue;
        end
        
        if No_image(m, n) > 0
            Label_image(m, n) = No_image(m, n);
        else
            idx_label = Assign(m, n);
            idx_x = ceil(double(idx_label) / double(n_height));
            idx_y = idx_label - n_height * (idx_x - 1);
            Label_image(m, n) = No_image(idx_y, idx_x);
        end
    end
end

% Label_show_image = 2 .* Label_image;
% Label_show_image = Label_show_image + 1 .*((Mask_image == 0) & (Label_image> 0));
% rgb = label2rgb(Label_show_image,'jet',[.5 .5 .5], 'shuffle');
% imshow(rgb)
% pause
% close all

thr_each = 256.0;
thr_all = 0.0;

for Inx = 1:num_particle
    All_Mask = (Label_image == Inx);
    Fore_Mask = All_Mask & (Mask_image > 0);
    Back_Mask = All_Mask & (Mask_image == 0);
    
    SE_All = var(Raw_image(All_Mask == 1)) * (sum(sum(All_Mask == 1)) - 1);
    SE_Fore = var(Raw_image(Fore_Mask == 1)) * (sum(sum(Fore_Mask == 1)) - 1);
    SE_Back = var(Raw_image(Back_Mask == 1)) * (sum(sum(Back_Mask == 1)) - 1);
    
    Ell_Score(Inx) = SE_All - SE_Fore - SE_Back;
    Ell_Score(Inx) = Ell_Score(Inx) - thr_each * double(sum(sum(All_Mask == 1)));
    
end

Ell_Score = Ell_Score / double(dia_particle * dia_particle);
Ell_Score = Ell_Score - thr_all;


