function [Ell_Para, Ell_Contour, Ell_Rel] = Ellipse_Fit(Re_image, dia_particle)
[n_height, n_width] = size(Re_image);

% Find the parameters of the fit of each ellipse
area_min = (dia_particle / 2)^2 / 2;
Bi_image2 = imcomplement(Re_image);
Bi_image2  = bwareaopen(Bi_image2 , round(area_min), 4);

Region = bwconncomp(Bi_image2, 4);
Stat_Region = regionprops(Region, 'Centroid', 'MajorAxisLength', 'MinorAxisLength', 'Orientation', 'BoundingBox',  'PixelList');
BoundingBox = cat(1, Stat_Region.BoundingBox);
region_max_height = max(BoundingBox(:, 4));
region_max_width = max(BoundingBox(:, 3));

edge_range = 20;
Temp_height = region_max_height + 2 * edge_range;
Temp_width = region_max_width + 2 * edge_range;
num_particles = length(Stat_Region);

Ell_Center = zeros(num_particles, 2);
Ell_Rel = zeros(num_particles, 1);

Temp_Ell = zeros(Temp_height, Temp_width);
Temp_Region_Bi = zeros(Temp_height, Temp_width);
num_fit = 0;

for i = 1:num_particles
    % The center of each ellipse
    Ell_Center(1) = Stat_Region(i).Centroid(1);
    Ell_Center(2) = Stat_Region(i).Centroid(2);
    
    Temp_Region_Bi = 0 .* Temp_Region_Bi;
    x_begin = floor(BoundingBox(i, 1));
    y_begin = floor(BoundingBox(i, 2));
    
    [n_row, n_col] = size(Stat_Region(i).PixelList);
    for k = 1:n_row
        Temp_Region_Bi(Stat_Region(i).PixelList(k, 2) - y_begin + edge_range, Stat_Region(i).PixelList(k, 1) - x_begin + edge_range) = 1;
    end
    
    % The contour of each ellise
    Temp_Ell = 0 .* Temp_Ell;
    X0_in = Ell_Center(1) - x_begin + edge_range;
    Y0_in = Ell_Center(2) - y_begin + edge_range;
    a = Stat_Region(i).MajorAxisLength / 2.0;
    b = Stat_Region(i).MinorAxisLength / 2.0;
    phi = Stat_Region(i).Orientation;
    phi = double(phi) / 180.0 * pi;
    
    orientation_rad = phi;
    cos_phi = cos( orientation_rad );
    sin_phi = sin( orientation_rad );
    R = [ cos_phi sin_phi; -sin_phi cos_phi];
    Center_or = R' * [X0_in; Y0_in];
    X0 = Center_or(1);
    Y0 = Center_or(2);
    
    for m = floor(Y0_in - 1.5 * a) : ceil(Y0_in + 1.5 * a)
        if m <= 1 || m >= Temp_height
            continue;
        end
        
        for n = floor(X0_in - 1.5 * a) : ceil(X0_in + 1.5 * a)
            if n <= 1 || n >= Temp_width
                continue;
            end
            
            cor_untiled = R' * [n; m];
            n_off = cor_untiled(1) - X0;
            m_off = cor_untiled(2) - Y0;
            
            r_ell = a^2* b^2 * (n_off^2 + m_off^2);
            dis_now = (n_off^2 + m_off^2) * (b^2 * n_off^2 + a^2 * m_off^2);
            
            if dis_now <= r_ell
                Temp_Ell(m, n) = 1;
            end
        end
    end
        
    
    Ell_Rel_now =  1 - sum(sum(Temp_Ell ~=  Temp_Region_Bi)) / max(sum(sum(Temp_Ell)), sum(sum(Temp_Region_Bi)));
    if Ell_Rel_now < 0
        Ell_Rel_now = 0;
    end
    
    if Ell_Rel_now < 0.3
        continue;
    end
    
    num_fit = num_fit + 1;
    Ell_Rel(num_fit) = Ell_Rel_now;
    Ell_Para(num_fit) = struct('x', Ell_Center(1), 'y', Ell_Center(2), 'a', a, 'b', b, 'phi', phi);
    
    Temp_Perimeter = bwboundaries(Temp_Ell, 8);
    Temp_Boundary = Temp_Perimeter{1};
    Temp_Boundary(:, 1) = Temp_Boundary(:, 1) - edge_range + y_begin;
    Temp_Boundary(:, 2) = Temp_Boundary(:, 2) - edge_range + x_begin;
    Ell_Contour(num_fit, 1) = {Temp_Boundary};
    
end
    

