function [Ell_Para_Best Ell_Contour_Best Ell_Rel_Best] = Evaluate_Result(Ell_Para_A, Ell_Contour_A, Ell_Para_B, Ell_Contour_B, Raw_image, dia_particle)

[n_height, n_width] = size(Raw_image);
Temp_image = zeros(n_height, n_width);

num_particle_A = length(Ell_Contour_A);
num_particle_B = length(Ell_Contour_B);

% fprintf('Calulation scores for different results...\n');
Ell_Score_A = Calculate_Score_Ell(Ell_Para_A, Ell_Contour_A, Raw_image, dia_particle);
Ell_Score_B = Calculate_Score_Ell(Ell_Para_B, Ell_Contour_B, Raw_image, dia_particle);

% Read the center of each particle
Ell_Center_A = zeros(num_particle_A, 2);
Ell_Center_B = zeros(num_particle_B, 2);

for i = 1:num_particle_A
    Ell_Center_A(i, 1) = Ell_Para_A(i).x;
    Ell_Center_A(i, 2) = Ell_Para_A(i).y;
end
for i = 1:num_particle_B
    Ell_Center_B(i, 1) = Ell_Para_B(i).x;
    Ell_Center_B(i, 2) = Ell_Para_B(i).y;
end
fprintf('The number of the detections via each method: %d (intensity-based), %d (gradient-based).\n', length(Ell_Contour_A), length(Ell_Contour_B));

Dist_A_B = sq_dist(Ell_Center_A',  Ell_Center_B');

% Compare the different results
num_neighbor_particle = 10;
Acc_Mat = zeros(num_particle_A, num_neighbor_particle);
Ind_Mat = zeros(num_particle_A, num_neighbor_particle);

for i = 1:num_particle_A
    [Min_Dis, Inx_Dis] = sort(Dist_A_B(i, :));
    Boundary_A = Ell_Contour_A{i};
    
    for j = 1:num_neighbor_particle
        Boundary_B = Ell_Contour_B{Inx_Dis(j)};
        Dist_Boundary = sq_dist(Boundary_A', Boundary_B');
        
        % Calculate the average distance between two boundaries
        Mean_Dist_A = mean(min(Dist_Boundary, [], 2));
        Mean_Dist_B = mean(min(Dist_Boundary, [], 1));
        Acc_Mat(i, j) = (Mean_Dist_A + Mean_Dist_B) / 2.0;
        Ind_Mat(i, j) = Inx_Dis(j);
    end 
end

Acc_Mat_Temp = reshape(Acc_Mat, num_particle_A * num_neighbor_particle, 1);
[Min_Acc, Inx_Acc] = sort(Acc_Mat_Temp);

b_Used_A = zeros(num_particle_A, 1);
b_Used_B = zeros(num_particle_B, 1);

% Find the match particles

num_match = 0;
for k = 1: num_particle_A * num_neighbor_particle
    Inx_now = Inx_Acc(k);
    i_now  = mod(Inx_now, num_particle_A);
    if i_now == 0
        i_now = num_particle_A;
    end
    j_now = floor((Inx_now - i_now) / num_particle_A + 1);
    
    % Test whether it is occupied
    if Acc_Mat(i_now, j_now) < 0
        continue;
    end
    
    
    % Calculate the relative threhold
    Inx_A = i_now;
    Inx_B = Ind_Mat(i_now, j_now);
    a = (Ell_Para_A(Inx_A).a + Ell_Para_B(Inx_B).a) / 2.0;
    b = (Ell_Para_A(Inx_A).b + Ell_Para_B(Inx_B).b) / 2.0;
    Thr_Acc = (a + b) / 2.0 * 0.5;
    
    if Acc_Mat(i_now, j_now) > Thr_Acc
        break;
    end
    
    num_match = num_match + 1;
    
    b_Used_A(Inx_A) = 1;
    b_Used_B(Inx_B) = 1;
    
    % Ell_Rel_Best(num_match) = 1 - Acc_Mat(i_now, j_now) / Thr_Acc;
    Ell_Rel_Best(num_match) = 1;
    
    % Set the parameter as the avarage of tow ellipses
    x = (Ell_Para_A(Inx_A).x + Ell_Para_B(Inx_B).x) / 2.0;
    y = (Ell_Para_A(Inx_A).y + Ell_Para_B(Inx_B).y) / 2.0;
    a = (Ell_Para_A(Inx_A).a + Ell_Para_B(Inx_B).a) / 2.0;
    b = (Ell_Para_A(Inx_A).b + Ell_Para_B(Inx_B).b) / 2.0;
    phi = (Ell_Para_A(Inx_A).phi + Ell_Para_B(Inx_B).phi) / 2.0;
    if abs(Ell_Para_A(Inx_A).phi - Ell_Para_B(Inx_B).phi) > pi / 2.0
        phi = phi + pi / 2.0;
    end
    
    Ell_Para_Best(num_match) = struct('x', x, 'y', y, 'a', a, 'b', b, 'phi', phi);
    
    % Draw the new contour
    Temp_image = 0 .* Temp_image;
    
    orientation_rad = phi;
    cos_phi = cos( orientation_rad );
    sin_phi = sin( orientation_rad );
    R = [cos_phi sin_phi; -sin_phi cos_phi];
    Center_or = R' * [x; y];
    X0 = Center_or(1);
    Y0 = Center_or(2);
    
    for m = floor(y - 1.5 * a) : ceil(y + 1.5 * a)
        if m <= 1 || m >= n_height
            continue;
        end
        
        for n = floor(x - 1.5 * a) : ceil(x + 1.5 * a)
            if n <= 1 || n >= n_width
                continue;
            end
            
            cor_untiled = R' * [n; m];
            n_off = cor_untiled(1) - X0;
            m_off = cor_untiled(2) - Y0;
            
            r_ell = a^2* b^2 * (n_off^2 + m_off^2);
            dis_now = (n_off^2 + m_off^2) * (b^2 * n_off^2 + a^2 * m_off^2);
            
            if dis_now <= r_ell
                Temp_image(m, n) = 1;
            end
        end
    end
    Temp_Perimeter = bwboundaries(Temp_image, 8);
    Temp_Boundary = Temp_Perimeter{1};
    Ell_Contour_Best(num_match) = {Temp_Boundary};
    
    % Removed the matched particles
    Acc_Mat(Inx_A, :) = -1;
    Acc_Mat(Ind_Mat == Inx_B) = -1;
    
end

fprintf('The number of the consensus detections: %d.\n', length(Ell_Contour_Best));
fprintf('The percentages of the consensus detections from each method: %5.2f%% (intensity-based), %5.2f%% (gradient-based).\n', double(length(Ell_Contour_Best))/double(length(Ell_Contour_A)) * 100.0, double(length(Ell_Contour_Best))/double(length(Ell_Contour_B)) * 100.0);
% return;

% Find the index of unused particles
Inx_Unused_A = find(b_Used_A == 0);
num_Unused_A = length(Inx_Unused_A);
Inx_Unused_B = find(b_Used_B == 0);
num_Unused_B = length(Inx_Unused_B);
Unused_Score_A = Ell_Score_A(Inx_Unused_A);
Unused_Score_B = Ell_Score_B(Inx_Unused_B);

% Find all the confilited particles
Conflict_Mat = zeros(num_Unused_A, num_Unused_B);
thr_overlapped = 0.2;
for i = 1:num_particle_A
    Inx_A = i;
    if b_Used_A(Inx_A) == 1
        continue;
    end
    
    % Draw the particles
    Temp_image = 0 .* Temp_image;
    x = Ell_Para_A(Inx_A).x;
    y = Ell_Para_A(Inx_A).y;
    a = Ell_Para_A(Inx_A).a;
    b = Ell_Para_A(Inx_A).b;
    phi = Ell_Para_A(Inx_A).phi;
    orientation_rad = phi;
    cos_phi = cos( orientation_rad );
    sin_phi = sin( orientation_rad );
    R = [ cos_phi sin_phi; -sin_phi cos_phi];
    Center_or = R' * [x; y];
    X0 = Center_or(1);
    Y0 = Center_or(2);
    Area_A = 0;
    for m = floor(y - 1.5 * a) : ceil(y + 1.5 * a)
        if m <= 1 || m >= n_height
            continue;
        end
        
        for n = floor(x - 1.5 * a) : ceil(x + 1.5 * a)
            if n <= 1 || n >= n_width
                continue;
            end
            
            cor_untiled = R' * [n; m];
            n_off = cor_untiled(1) - X0;
            m_off = cor_untiled(2) - Y0;
            
            r_ell = a^2* b^2 * (n_off^2 + m_off^2);
            dis_now = (n_off^2 + m_off^2) * (b^2 * n_off^2 + a^2 * m_off^2);
            
            if dis_now <= r_ell
                Temp_image(m, n) = 1;
                Area_A = Area_A + 1;
            end
        end
    end
    
    
    for j = 1:num_neighbor_particle
        
        Inx_B = Ind_Mat(i, j);
        if b_Used_B(Inx_B) == 1
            continue;
        end
        
        % Calculate the overlapped pixels
        num_overlapped = 0;
        x = Ell_Para_B(Inx_B).x;
        y = Ell_Para_B(Inx_B).y;
        a = Ell_Para_B(Inx_B).a;
        b = Ell_Para_B(Inx_B).b;
        phi = Ell_Para_B(Inx_B).phi;
        orientation_rad = phi;
        cos_phi = cos( orientation_rad );
        sin_phi = sin( orientation_rad );
        R = [ cos_phi sin_phi; -sin_phi cos_phi];
        Center_or = R' * [x; y];
        X0 = Center_or(1);
        Y0 = Center_or(2);
        Area_B = 0;
        for m = floor(y - 1.5 * a) : ceil(y + 1.5 * a)
            if m <= 1 || m >= n_height
                continue;
            end
            
            for n = floor(x - 1.5 * a) : ceil(x + 1.5 * a)
                if n <= 1 || n >= n_width
                    continue;
                end
                
                cor_untiled = R' * [n; m];
                n_off = cor_untiled(1) - X0;
                m_off = cor_untiled(2) - Y0;
                
                r_ell = a^2* b^2 * (n_off^2 + m_off^2);
                dis_now = (n_off^2 + m_off^2) * (b^2 * n_off^2 + a^2 * m_off^2);
                
                if dis_now <= r_ell
                    if Temp_image(m, n) == 1
                        num_overlapped = num_overlapped + 1;
                    end
                    Area_B = Area_B + 1;
                end
            end
        end
        rate_overlapped = double(num_overlapped) / double(min(Area_A, Area_B));
        
        if rate_overlapped >= thr_overlapped
            Inx_A_new = find(Inx_Unused_A == Inx_A, 1, 'first');
            Inx_B_new = find(Inx_Unused_B == Inx_B, 1, 'first');
            Conflict_Mat(Inx_A_new, Inx_B_new) = 1;
        end
    end 
end

% Use Integer Optimization to solve this problem
f_op = (-1) .* [Unused_Score_A Unused_Score_B]';
f_scale = max(abs(f_op));
f_scale = power(10.0, floor(log10(f_scale)));
f_op = f_op ./ double(f_scale);

% fprintf('Optimization begins...\n');

% Sparate the graph in to subgraph
Conflict_Mat_All = [eye(num_Unused_A) Conflict_Mat;
    Conflict_Mat' eye(num_Unused_B)];
Conflict_Mat_D = diag(sum(Conflict_Mat_All));
Conflict_Mat_Lap = Conflict_Mat_D - Conflict_Mat_All;

err = 1e-2;
[V,D] = eig(Conflict_Mat_Lap);
d = diag(D);
inx_d = find(abs(d) < err);
b_Unused_All = ones(num_Unused_A + num_Unused_B, 1);

% Solve the binary integer programming by 
x_op = zeros(num_Unused_A + num_Unused_B, 1);
options = optimoptions('intlinprog', 'Display', 'off');
for k = 1:length(inx_d)
    
    [max_num_v, max_idx_v] = max(abs(V(:, inx_d(k))) .* b_Unused_All);
    if max_num_v < 1e-10
        continue;
    end
    Indicator_All = V(:, inx_d(k)) .* b_Unused_All .* sign(V(max_idx_v, inx_d(k)));
    Set_All = find(Indicator_All >= (1 - err) * max_num_v);
    b_Unused_All(Set_All) = 0;
    
    num_Set_A = length(find(Set_All <= num_Unused_A));
    num_Set_B = length(Set_All) - num_Set_A;
    
    f_op_Set = f_op(Set_All);
    if num_Set_A == 0 || num_Set_B == 0
        x_op_Set = intlinprog(f_op_Set, 1:(num_Set_A + num_Set_B),[],[],[],[],zeros(num_Set_A + num_Set_B, 1), ones(num_Set_A + num_Set_B, 1), options); % The optimization without constrainst
    else
        Set_A = Set_All(1:num_Set_A);
        Set_B = Set_All(num_Set_A + 1:num_Set_A + num_Set_B) - num_Unused_A;
        
        Conflict_Mat_Set = Conflict_Mat(Set_A, Set_B);
        A_op_Set = [Conflict_Mat_Set' num_Set_A .* eye(num_Set_B)];
        b_op_Set = (num_Set_A + 0.5) .* ones(num_Set_B, 1);
        x_op_Set = intlinprog(f_op_Set, 1:(num_Set_A + num_Set_B), A_op_Set, b_op_Set,[],[],zeros(num_Set_A + num_Set_B, 1), ones(num_Set_A + num_Set_B, 1), options); % The optimization with constrainst
    end
    
    x_op(Set_All) = x_op_Set;
end

% options = optimset('NodeSearchStrategy','df','MaxTime',2000, 'TolXInteger', 1.e-5);
% x_op = bintprog(f_op, A_op, b_op,[],[], x0_op, options);


b_choice_A = x_op(1:num_Unused_A);
b_choice_B = x_op(num_Unused_A + 1:num_Unused_A + num_Unused_B);

num_choice_A = 0;
num_choice_B = 0;
for i = 1:num_Unused_A
    if b_choice_A(i) == 1
        Inx_A = Inx_Unused_A(i);
        num_match = num_match + 1;
        num_choice_A = num_choice_A + 1;
        Ell_Rel_Best(num_match) = 0; % Choose the method A
        Ell_Para_Best(num_match) = Ell_Para_A(Inx_A);
        Ell_Contour_Best(num_match) = {Ell_Contour_A{Inx_A}};
    end
end

for i = 1:num_Unused_B
    if b_choice_B(i) == 1
        Inx_B = Inx_Unused_B(i);
        num_match = num_match + 1;
        num_choice_B = num_choice_B + 1;
        Ell_Rel_Best(num_match) = 2; % Choose the method B
        Ell_Para_Best(num_match) = Ell_Para_B(Inx_B);
        Ell_Contour_Best(num_match) = {Ell_Contour_B{Inx_B}};
    end
end
    

fprintf('The number of detections choosen from the results of intensity-based method: %d (%5.2f%%).\n', num_choice_A, double(num_choice_A) / double(num_choice_A + num_choice_B) * 100.0);
fprintf('The number of detections choosen from the results of gradient-based method: %d (%5.2f%%).\n', num_choice_B, double(num_choice_B) / double(num_choice_A + num_choice_B) * 100.0);
fprintf('The number of particles in the combined results: %d.\n', length(Ell_Contour_Best));





