function [Re_image] = seg_particle_Convex_Park(Bi_image, Raw_image, dia_particle)

[n_height n_width] = size(Raw_image);

% Saperte the particles block by block
area_min = (dia_particle / 2)^2 / 2;
Bi_image2 = imcomplement(Bi_image);
Bi_image2  = bwareaopen(Bi_image2 , round(area_min));
Re_image = imcomplement(Bi_image2);

Region = bwconncomp(Bi_image2);
Stat_Region = regionprops(Region,'BoundingBox','PixelList');
% Find the largest
BoundingBox = cat(1, Stat_Region.BoundingBox);
region_max_height = max(BoundingBox(:, 4));
region_max_width = max(BoundingBox(:, 3));

edge_range = 5;
Temp_height = region_max_height + 2 * edge_range;
Temp_width = region_max_width + 2 * edge_range;
Temp_Region_Bi = zeros(Temp_height, Temp_width);

Temp_Count = zeros(Temp_height, Temp_width);

pr_erode = 3;
se1    = strel('disk', 1 );
se2    = strel('arbitrary', ones(3,3));

for i = 1:length(Stat_Region)
    Temp_Region_Bi = 0 .* Temp_Region_Bi;
    x_begin = floor(BoundingBox(i, 1));
    y_begin = floor(BoundingBox(i, 2));
    
    [n_row n_col] = size(Stat_Region(i).PixelList);
    for k = 1:n_row
        Temp_Region_Bi(Stat_Region(i).PixelList(k, 2) - y_begin + edge_range, Stat_Region(i).PixelList(k, 1) - x_begin + edge_range) = 1;
    end
    
    % Copy each region
    
    Temp_Ero = imerode(Temp_Region_Bi, strel('disk', pr_erode));
    Temp_Count = 0 .* Temp_Count;
    
    updated = 1; cnt = 0;
    
    while updated
        updated = 0;
        [B, L] = bwboundaries(Temp_Ero,8,'noholes');
        s  = regionprops(L, 'ConvexArea', 'Area', 'Orientation', 'MajorAxisLength', 'MinorAxisLength');
        if mod(cnt, 2) == 1
            se = se1;
        else
            se = se2;
        end
        
        for j = 1:size(B,1)
            pts = B{j,1};
            pls = logical(L==j);
            cc = concavity(pts);
            % Calculate the concavity of each marker
            if (max(cc) > 0.2|| 1 - s(j).Area / s(j).ConvexArea > 0.2)&& s(j).MinorAxisLength > floor(dia_particle / 10.0)
                tmp = imerode(pls, se);
                tmp = imopen(tmp, se1);
                updated = 1;
                Temp_Ero(pls) = tmp(pls);
            else
                if max(unique(Temp_Count(pls))) == 0
                    Temp_Count(pls) = cnt + 1;
                end
            end
        end
        cnt = cnt + 1;
    end
    cnt_total = cnt;
    
    if sum(sum(Temp_Ero)) == 0
        continue;
    end
    
    % Now the Temp_Ero is the marker image, we can use it to separte
    [B, L] = bwboundaries(Temp_Ero,8,'noholes');
    num_markers = size(B,1);
    
    if num_markers == 1
        continue;
    end
    
    Temp_Label = L;
    updated = 1; cnt = 0;
    while updated 
        if mod(cnt, 2) == 1
            se = se1;
        else
            se = se2;
            updated = 0;
        end
        
        for j = 1:num_markers
            
            pls = logical(Temp_Label==j);
            % Read the number of erosion
            cnt_ero = max(unique(Temp_Count(pls)));
            
            % The thought of Multi-stage
            if (cnt + 1) < (cnt_total - cnt_ero)
                continue;
            end
            
            tmp = imdilate(pls, se);
            tmp = imclose(tmp, se1);
            
            tmp2 = tmp & (Temp_Label == 0) & (Temp_Region_Bi > 0);
            if sum(sum(tmp2)) > 0 
                updated = 1;
            end
            tmp = tmp2 | pls;
            Temp_Label(tmp == 1) = j;
        end
       
        cnt = cnt + 1;
        
    end
    
    for j = 1:num_markers
        Temp_Saperated = (Temp_Label == j);
        
        if sum(sum(Temp_Saperated)) == 0
            continue;
        end
        
        Temp_CC = bwconncomp(Temp_Saperated, 4);
        Temp_statall = regionprops(Temp_CC, 'Area');
        Temp_Area = cat(1, Temp_statall.Area);
        [max_Area, max_idx] = max(Temp_Area);
        Temp_Perimeter = bwboundaries(Temp_Saperated, 4);
        
        Temp_Boundary = Temp_Perimeter{max_idx};
        
        for k = 1 : length(Temp_Boundary)
            
            % Find the separating boundary
            Neighbor_Label = zeros(1,4);
            Neighbor_Label(1) = Temp_Label(Temp_Boundary(k, 1) - 1, Temp_Boundary(k, 2));
            Neighbor_Label(2) = Temp_Label(Temp_Boundary(k, 1) + 1, Temp_Boundary(k, 2));
            Neighbor_Label(3) = Temp_Label(Temp_Boundary(k, 1), Temp_Boundary(k, 2) - 1);
            Neighbor_Label(4) = Temp_Label(Temp_Boundary(k, 1), Temp_Boundary(k, 2) + 1);
            if sum(Neighbor_Label ~= 0 & Neighbor_Label ~= j) == 0
                continue;
            end
            
            y_org = Temp_Boundary(k, 1) - edge_range + y_begin;
            x_org = Temp_Boundary(k, 2) - edge_range + x_begin;
            
            if y_org < 1 || y_org > n_height || x_org < 1 || x_org > n_width
                continue;
            end
            
            Re_image(y_org, x_org) = 1;
        end
        
    end
end

% Remove too small particles
Re_image2 = imcomplement(Re_image);
Re_image2  = bwareaopen(Re_image2 , round(area_min), 4);
Re_image = imcomplement(Re_image2);
